#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
THIS FILE IS PART OF AZUR LANE TOOL BY MATT BELFAST BROWN
mode_BlP_Cal.py - The core mode of the Azur Lane Tool.

Author: Matt Belfast Brown
Create Date: 2021-07-10
Version Date: 2022-02-22
Version: 0.5.1
Mode Create Date: 2020-05-02
Mode Date: 2022-02-15
Mode Version: 1.1.2

THIS PROGRAM IS FREE FOR EVERYONE,IS LICENSED UNDER GPL-3.0
YOU SHOULD HAVE RECEIVED A COPY OF GPL-3.0 LICENSE.

Copyright (C) 2021-2022 Matt Belfast Brown

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


# define function list
def fun_cnbp_rqub(vari_plle):
    """
    此函数输入科研等级达到对应等级所需的蓝图数量。
    This function inputs the number of blueprints required to reach the corresponding level of scientific research.
    :param vari_plle: 变量:科研等级 <Plan Level>
    :return vari_bprb: 变量:升级所需基础蓝图数量 <Number of Basic Blueprint which Required for Upgrade>
    """
    if 1 <= vari_plle <= 4 or 6 <= vari_plle <= 9 or 11 <= vari_plle <= 14:
        vari_bprb = 0.4 * vari_plle - 0.4 * (vari_plle % 5) + 2
    elif 16 <= vari_plle <= 19 or 21 <= vari_plle <= 24 or 26 <= vari_plle <= 29:
        vari_bprb = vari_plle - (vari_plle % 5) - 5
    elif vari_plle == 5:
        vari_bprb = 5
    elif vari_plle % 5 == 0:
        vari_bprb = int(-(vari_plle ** 3) / 375 + (vari_plle ** 2) / 5 - 2.9333 * vari_plle + 20)
    else:
        vari_bprb = 0
    return int(vari_bprb)


def fun_cnbp_rqup(flag_pltp, vari_plle):
    """
    此函数输入舰船科研类别与科研等级,输出升级所需蓝图数量.
    This function inputs the research type and research level of the ship, and outputs the number of blueprints required for the upgrade.
    :param flag_pltp: 舰船科研类别 包括:'Top Solution'(最高方案)和'Decisive Plan'(决战方案)两个参数.
    The ship research category includes: 'Top Solution' and 'Decisive Plan' parameters.
    :param vari_plle: 变量:科研等级 <Plan Level>
    :return vari_bprq:变量:升级所需蓝图数量 <Number of Blueprint which Required for Upgrade>
    """
    if flag_pltp == 'Top Solution':
        vari_bprq = fun_cnbp_rqub(vari_plle)
    elif flag_pltp == 'Decisive Plan':
        vari_bprq = int(fun_cnbp_rqub(vari_plle) * 1.5)
    else:
        vari_bprq = 0
    return vari_bprq


def fun_cnbp_rrcl(flag_pltp, vari_plde, vari_epbp, vari_lede, vari_levn):
    """
    此函数输入下列五个参数输出非天运科研图纸需求数量以及已用图纸数量。
    This function inputs the following five parameters and outputs the required quantity of non-Tianyun scientific research drawings and the quantity of used blueprints.
    :param flag_pltp: 舰船科研类别 包括:'Top Solution'(最高方案)和'Decisive Plan'(决战方案)两个参数.
    The ship research category includes: 'Top Solution' and 'Decisive Plan' parameters.
    :param vari_plde: 已拥有未使用的蓝图数量. Already have the number of unused blueprints.
    :param vari_epbp: 本级已使用的蓝图数量. The number of blueprints used by this level.
    :param vari_lede: 需要升到的科研等级. The plan level which need.
    :param vari_levn: 目前的科研等级. The plan leve which had.
    :return vari_prbp, vari_tebp:  科研图纸需求计算结果以及已用的总蓝图数.
    Calculation results of scientific blueprint requirements and the total number of blueprints used.
    """
    vari_tbpr = 0
    vari_tebp = 0
    for para_plle in range(vari_lede):
        vari_tbpr += fun_cnbp_rqup(flag_pltp, para_plle)
    for para_plee in range(vari_levn):
        vari_tebp += fun_cnbp_rqup(flag_pltp, para_plee)
    vari_tebp += vari_epbp
    vari_prbp = vari_tbpr - vari_plde - vari_tebp
    return vari_prbp, vari_tebp


def fun_cnbp_tyfi(flag_pltp, vari_tfdl, vari_tyfg, vari_crtf):
    """
    此函数输入下列四个参数输出天运拟合总需蓝图数与天运拟合总用蓝图数.
    This function inputs the following four parameters and outputs the total number of blueprints required for the fitting of the sky and the total number of blueprints used for the fitting of the sky.
    :param flag_pltp: 舰船科研类别 包括:'Top Solution'(最高方案)和'Decisive Plan'(决战方案)两个参数.
    The ship research category includes: 'Top Solution' and 'Decisive Plan' parameters.
    :param vari_tfdl: 天运拟合现有等级. Tianyun fits existing grades.
    :param vari_tyfg: 天运拟合未使用蓝图数量. Number of unused blueprints.
    :param vari_crtf: 天运拟合目前经验值. Tianyun fits the current experience value.
    :return vari_tbtf, vari_bpty: 天运拟合总需蓝图数，天运拟合总用蓝图数
    The total number of blueprints required for Tianyun fitting. The total number of blueprints required for Tianyun fitting.
    """
    vari_bpty = 0
    if flag_pltp == 'Top Solution':
        for i in range(vari_tfdl):
            vari_bpty += list_fitt[i]
        vari_bpty += int((vari_crtf / 100) * list_fitt[vari_tfdl])
        vari_tbtf = 165 - vari_tyfg - vari_bpty
    elif flag_pltp == 'Decisive Plan':
        for i in range(vari_tfdl):
            vari_bpty += list_fitd[i]
        vari_bpty += int((vari_crtf / 100) * list_fitt[vari_tfdl])
        vari_tbtf = 215 - vari_tyfg - vari_bpty
    else:
        vari_tbtf = 0
    return vari_tbtf, vari_bpty


def fun_cnbp_rbpt(flag_pltp, flag_pftf, vari_lede, vari_levn, vari_plde, vari_epbp, vari_tfdl, vari_tyfg, vari_crtf):
    """
    此函数是集合函数，聚合上述函数内容的综合型函数.
    This function is an aggregate function, a comprehensive function that aggregates the contents of the above functions.
    :param flag_pltp: 舰船科研类别 包括:'Top Solution'(最高方案)和'Decisive Plan'(决战方案)两个参数.
    The ship research category includes: 'Top Solution' and 'Decisive Plan' parameters.
    :param flag_pftf: 是否计算天运拟合，包括 True 和 False 两个参数。
    Whether to calculate the luck fit, including two parameters True and False.
    :param vari_lede: 需要升到的科研等级. The plan level which need.
    :param vari_levn: 目前的科研等级. The plan leve which had.
    :param vari_plde: 已拥有未使用的蓝图数量. Already have the number of unused blueprints.
    :param vari_epbp: 本级已使用的蓝图数量. The number of blueprints used by this level.
    :param vari_tfdl: 天运拟合现有等级. Tianyun fits existing grades.
    :param vari_tyfg: 天运拟合未使用蓝图数量. Number of unused blueprints.
    :param vari_crtf: 天运拟合目前经验值. Tianyun fits the current experience value.
    :return [vari_prbt, vari_tbpt, vari_prbp, vari_tebp, vari_tbtf,
            vari_bpty]: 列表，每项内容如下：[科研图纸需求含天运拟合,已用的总蓝图数含天运拟合,科研图纸需求计算结果,已用的总蓝图数,天运拟合总需蓝图数,天运拟合总用蓝图数]。
            list, each of which is as follows: [Scientific research drawing requirements include Tianyun fitting, the total number of blueprints used includes Tianyun fitting, the calculation result of scientific research drawing requirements, the total number of blueprints used, the total number of blueprints required for Tianyun fitting, and the total number of Tianyun fitting number of blueprints].
    """
    if flag_pftf:
        vari_tbtf, vari_bpty = fun_cnbp_tyfi(flag_pltp, vari_tfdl, vari_tyfg, vari_crtf)
    else:
        vari_tbtf, vari_bpty = 0, 0
    vari_prbp, vari_tebp = fun_cnbp_rrcl(flag_pltp, vari_plde, vari_epbp, vari_lede, vari_levn)
    vari_prbt = vari_prbp + vari_tbtf
    vari_tbpt = vari_tebp + vari_bpty
    return [vari_prbt, vari_tbpt, vari_prbp, vari_tebp, vari_tbtf,
            vari_bpty]


# define list
list_fitt = [10, 20, 30, 40, 65]
list_fitd = [20, 30, 40, 50, 75]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
THIS FILE IS PART OF AZUR LANE TOOL BY MATT BELFAST BROWN
__init__.py - The core part of the Azur Lane Tool.

Author: Matt Belfast Brown
Create Date: 2021-07-10
Version Date: 2022-02-22
Version: 0.5.1

THIS PROGRAM IS FREE FOR EVERYONE,IS LICENSED UNDER GPL-3.0
YOU SHOULD HAVE RECEIVED A COPY OF GPL-3.0 LICENSE.

Copyright (C) 2021-2022 Matt Belfast Brown

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__title__ = 'AzurLaneToolLib.mode'
__version__ = '0.5.1'
__author__ = 'Matt Belfast Brown'
__license__ = 'GPL-3.0'
__copyright__ = 'Copyright (c) 2020-2022 Matt Belfast Brown'
__all__ = ['mode_EXP_Cal', 'mode_BlP_Cal', 'mode_KSN_Com', 'mode_FCS_Cal', 'mode_CME_Cal', 'mode_FLE_Tol']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
THIS FILE IS PART OF AZUR LANE TOOL BY MATT BELFAST BROWN
mode_EXP_Cal.py - The core mode of the Azur Lane Tool.

Author: Matt Belfast Brown
Create Date: 2021-07-10
Version Date: 2022-02-22
Version: 0.5.1
Mode Create Date: 2020-09-25
Mode Date: 2022-02-15
Mode Version: 1.0.2

THIS PROGRAM IS FREE FOR EVERYONE,IS LICENSED UNDER GPL-3.0
YOU SHOULD HAVE RECEIVED A COPY OF GPL-3.0 LICENSE.

Copyright (C) 2021-2022 Matt Belfast Brown

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


# define function
def fun_cexp_vrfu(flag_kstp: str, vari_leve: int) -> int:
    """
    此函数输入 <舰船类别> `flag_kstp` 与 <等级变量> `vari_leve` 得到该等级舰船 <升至下一级所需经验值> `vari_erfv`。
    数学运算公式为：int(1000 * vari_vara * vari_leve - 100 * vari_inva)
    '''
    This function inputs <ship type> `flag_kstp` and <level variable> `vari_leve` to get the <experience value required to advance to the next level> `vari_erfv` for ships of this level.
    The mathematical formula is: int(1000 * vari_vara * vari_leve - 100 * vari_inva)
    Translated by Google
    '''
    """
    if flag_kstp == 'Top Solution':
        if vari_leve == 0:
            vari_vara = 0
            vari_inva = 0
        elif 0 < vari_leve <= 40:
            vari_vara = 0.1
            vari_inva = 0
        elif 40 < vari_leve <= 60:
            vari_vara = 0.2
            vari_inva = 40
        elif 60 < vari_leve <= 70:
            vari_vara = 0.3
            vari_inva = 100
        elif 70 < vari_leve <= 80:
            vari_vara = 0.4
            vari_inva = 170
        elif 80 < vari_leve <= 90:
            vari_vara = 0.5
            vari_inva = 250
        elif 90 < vari_leve <= 92:
            vari_vara = 1
            vari_inva = 700
        elif 92 < vari_leve <= 94:
            vari_vara = 2
            vari_inva = 1620
        elif vari_leve == 95:
            vari_vara = 4
            vari_inva = 3500
        elif 95 < vari_leve <= 97:
            vari_vara = 5
            vari_inva = 4450
        elif vari_leve == 98:
            vari_vara = 20
            vari_inva = 19000
        elif vari_leve == 99:
            vari_vara = 72
            vari_inva = 69960
        elif vari_leve == 100:
            vari_vara = -82
            vari_inva = -82500
        elif 100 < vari_leve <= 105:
            vari_vara = 3
            vari_inva = 2500
        elif 105 < vari_leve <= 110:
            vari_vara = 6
            vari_inva = 5650
        elif 110 < vari_leve <= 115:
            vari_vara = 10
            vari_inva = 10050
        elif 115 < vari_leve <= 120:
            vari_vara = 15
            vari_inva = 15800
        elif 120 < vari_leve < 125:
            vari_vara = 21
            vari_inva = 23000
        elif vari_leve == 125:
            vari_vara = 2696
            vari_inva = 3340000
        else:
            vari_vara = 0
            vari_inva = 0
    elif flag_kstp == 'Decisive Plan':
        if vari_leve == 0:
            vari_vara = 0
            vari_inva = 0
        elif 0 < vari_leve <= 40:
            vari_vara = 0.12
            vari_inva = 0
        elif 40 < vari_leve <= 60:
            vari_vara = 0.24
            vari_inva = 48
        elif 60 < vari_leve <= 70:
            vari_vara = 0.36
            vari_inva = 120
        elif 70 < vari_leve <= 80:
            vari_vara = 0.48
            vari_inva = 204
        elif 80 < vari_leve < 90:
            vari_vara = 0.6
            vari_inva = 300
        elif vari_leve == 90:
            vari_vara = 0.65
            vari_inva = 325
        elif 90 < vari_leve <= 92:
            vari_vara = 1.3
            vari_inva = 910
        elif 92 < vari_leve <= 94:
            vari_vara = 2.6
            vari_inva = 2106
        elif vari_leve == 95:
            vari_vara = 5.2
            vari_inva = 4550
        elif 95 < vari_leve <= 97:
            vari_vara = 6.5
            vari_inva = 5785
        elif vari_leve == 98:
            vari_vara = 26
            vari_inva = 24700
        elif vari_leve == 99:
            vari_vara = 93.6
            vari_inva = 90948
        elif vari_leve == 100:
            vari_vara = -111.6
            vari_inva = -112200
        elif 100 < vari_leve <= 105:
            vari_vara = 3.6
            vari_inva = 3000
        elif 105 < vari_leve <= 110:
            vari_vara = 7.2
            vari_inva = 6780
        elif 110 < vari_leve <= 115:
            vari_vara = 12
            vari_inva = 12060
        elif 115 < vari_leve <= 120:
            vari_vara = 18
            vari_inva = 18960
        elif 120 < vari_leve < 125:
            vari_vara = 25.2
            vari_inva = 27600
        elif vari_leve == 125:
            vari_vara = 2635.2
            vari_inva = 3264000
        else:
            vari_vara = 0
            vari_inva = 0
    else:
        vari_vara = 0
        vari_inva = 0
    cons_slin = 1000
    cons_inqu = -100
    para_ince = cons_inqu * vari_inva
    para_slop = cons_slin * vari_vara
    vari_erfv = int(para_slop * vari_leve + para_ince)
    return vari_erfv


def fun_crex_cele(flag_kstp: str, vari_lede: int, vari_levn: int, vari_eexp: int) -> tuple[int, int]:
    """
    计算舰船升至某级所需经验值的函数，输入如下四个变量 <舰船类别> `flag_kstp` <舰船目前等级> `vari_lede` <舰船所需等级> `vari_levn` <舰船已有经验值> `vari_eexp` 输出两个变量 <舰船总需经验值> `vari_rexp` <舰船现有总经验值> `vari_teep` 。
    '''
    A function to calculate the experience value required for a ship to upgrade to a certain level, enter the following four variables <ship type> `flag_kstp` <ship's current level> `vari_lede` <ship required level> `vari_levn` <ship has With experience> `vari_eexp` outputs two variables <ship total experience required> `vari_rexp` <ship existing total experience> `vari_teep`.
    Translated by Google
    '''
    """
    vari_texp = 0
    vari_teep = 0
    for para_leve in range(vari_lede):
        vari_texp += fun_cexp_vrfu(flag_kstp, para_leve)
    for para_leex in range(vari_levn):
        vari_teep += fun_cexp_vrfu(flag_kstp, para_leex)
    vari_teep += vari_eexp
    vari_rexp = vari_texp - vari_teep
    return vari_rexp, vari_teep



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
THIS FILE IS PART OF AZUR LANE TOOL BY MATT BELFAST BROWN
mode_FCS_Cal.py - The core mode of the Azur Lane Tool.

Author: Matt Belfast Brown
Create Date: 2021-07-10
Version Date: 2022-02-22
Version: 0.5.1
Mode Create Date: 2020-05-27
Mode Date: 2022-02-22
Mode Version: 0.5.0

THIS PROGRAM IS FREE FOR EVERYONE,IS LICENSED UNDER GPL-3.0
YOU SHOULD HAVE RECEIVED A COPY OF GPL-3.0 LICENSE.

Copyright (C) 2021-2022 Matt Belfast Brown

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
#define variable name
 variable-name chinese
 fun_sfks_fuca 函数-计算水面舰船油耗
 fun_smks_fuca 函数-计算潜艇油耗
 fun_bsdg_fuca 函数-计算基础满级油耗
 fun_smks_mfuc 函数-计算潜艇移动油耗
 vari_ksct     变量-舰船类别
 para_ksct     参数-舰船类别
 
 #
 DD：驱逐；CL：轻巡；CA：重巡；MN：浅水重炮舰；MT：维修舰；
 CVL：轻航；CV：航母；BC：战列巡洋舰；BB：战列舰 
 #
 
'''


# define import list

# define function
def fun_bsdg_fuca(vari_ksct: str, vari_rare: str, vari_ksnm: str) -> int:
    """

    """
    # 计算舰船类型补正 - para_ksct
    if vari_ksct == 'DD':
        para_ksct = 0
    elif vari_ksct in ['CL', 'MN', 'MT']:
        para_ksct = 1
    elif vari_ksct in ['CA', 'CVL']:
        para_ksct = 2
    elif vari_ksct == 'CV':
        para_ksct = 3
    elif vari_ksct == 'BC':
        para_ksct = 4
    elif vari_ksct == 'BB':
        para_ksct = 5
    else:
        para_ksct = 0
    # 计算稀有度补正 - para_raco
    if vari_rare == '普通':
        para_raco = 0
    elif vari_rare == '稀有':
        para_raco = 1
    elif vari_rare == '精锐':
        para_raco = 2
    elif vari_rare in ['超稀有', '最高方案']:
        para_raco = 3
    elif vari_rare in ['海上传奇', '决战方案']:
        para_raco = 4
    else:
        para_raco = 0
    vari_scco = fun_kstp_scco(vari_ksnm)
    vari_bofb = list_bofd[para_ksct]  # 船型基础油耗
    vari_raco = list_raco[para_raco]  # 稀有度油耗
    vari_dgfu = int(vari_bofb + vari_raco + vari_scco)  # 基础最终油耗（100级0破）
    return vari_dgfu


def fun_sfks_fuca(vari_brea: int, vari_ksct: str, vari_rare: str, vari_ksnm: str, vari_kslv: int) -> int:
    """

    """
    # 计算突破补正 - para_brco
    if vari_brea == 0:
        para_brco = 0
    elif vari_brea == 1:
        para_brco = 1
    elif vari_brea == 2:
        para_brco = 2
    elif vari_brea == 3:
        para_brco = 3
    else:
        para_brco = 0
    vari_brco = list_brco[para_brco]  # 突破度油耗
    vari_dgfu = fun_bsdg_fuca(vari_ksct, vari_rare, vari_ksnm)
    vari_sfkf = int(int(1 + (vari_dgfu * (0.5 + min(vari_kslv, 99)) * 0.05)) + vari_brco)
    return vari_sfkf


def fun_kstp_scco(para_ksnm: str) -> int:
    """

    """
    # 计算特殊补正
    if para_ksnm in ['夕张']:
        vari_scco = list_scco[0]
    elif para_ksnm in ['神风', '多塞特郡', '最上', '三隈', '三笠']:
        vari_scco = list_scco[1]
    elif para_ksnm in ['飞鹰', '隼鹰', '半人马', '金刚', '比叡', '天城']:
        vari_scco = list_scco[2]
    else:
        vari_scco = 0
    return vari_scco


# define list
list_bofd = [1, 2, 3, 4, 5, 6]
list_raco = [0, 1, 2, 3, 4]
list_brco = [0, 2, 4, 6]
list_spco = [7, 9]
list_scco = [-2, -1, 1]
list_scks = ['夕张', '神风', '多塞特郡', '最上', '三隈', '三笠', '飞鹰', '隼鹰', '半人马', '金刚', '比叡', '天城']
